@@ -45,7 +45,7 @@ def _queryLogData(logId):
     logbody = restClient.callAPI(action)
     return logRecord,logbody
 
-def _parseStorage(error=False):
+def _parseStorage(error=False,printLimits=False):
     search_dir = restClient.logFolder()
 
     os.chdir(search_dir)
@@ -55,10 +55,12 @@ def _parseStorage(error=False):
     fileNames = [os.path.basename(f) for f in files]
     ids = [f.split('.')[0] for f in fileNames]
     print(f"Parsing {len(ids)} files in {search_dir}")
-    for id in ids:
-        parseLog(logId=id,error=error)
+    for num,id in enumerate(ids):
+        sys.stdout.write("\r%d%%" % int(100*num/len(ids)))
+        res = parseLog(logId=id,error=error,printLimits=printLimits)
+        
 
-def _parseTail(loguser=None,level=None,whereClause=None,deleteLogs=True):
+def _parseTail(loguser=None,level=None,whereClause=None,deleteLogs=True,printLimits=False):
     deleteList = []
     def deleteRecords(q):
       while True:
@@ -105,7 +107,7 @@ def _parseTail(loguser=None,level=None,whereClause=None,deleteLogs=True):
 
                 for record in logRecords:
 
-                    parseLog(logId=record['Id'], loguser=loguser,level=level)                
+                    parseLog(logId=record['Id'], loguser=loguser,level=level,printLimits=printLimits)                
                     _time = record[timefield]
                     _timez = _time.split('.')[0] + "Z"
                     if q!=None:
@@ -156,14 +158,14 @@ def _getLogUserId(loguser):
         utils.raiseException('QUERY',f"User with field {key} = {value} does not exist in the User Object.")    
     return id
 
-def parseLog(logId=None,filepath=None,printLimits=True,lastN=1,loguser=None,level=None,whereClause=None,writeToFile=False,tail=False,deleteLogs=False,store=False,error=False):
+def parseLog(logId=None,filepath=None,printLimits=False,lastN=1,loguser=None,level=None,whereClause=None,writeToFile=False,tail=False,deleteLogs=False,store=False,error=False):
     body = None
 
     if tail == True:
         return _parseTail(loguser=loguser,level=level,whereClause=whereClause,deleteLogs=deleteLogs)
 
     elif store == True:
-        return _parseStorage(error=error)
+        return _parseStorage(error=error,printLimits=printLimits)
 
     elif filepath != None:
         body = file.read(filepath)
@@ -190,10 +192,12 @@ def parseLog(logId=None,filepath=None,printLimits=True,lastN=1,loguser=None,leve
         filename = f"{restClient.logFolder()}{logId}.log"
         file.write(filename,body)
 
-    return parse(logId,body,printLimits=printLimits,level=level,writeTofile=writeToFile,error=error)
+    return parse(logId,body,printLimits=printLimits,level=level,writeTofile=writeToFile,error=error,limits=printLimits)
 
 def printLogRecords(loguser=None,limit=50,whereClause=None):
     logUserId = _getLogUserId(loguser) if loguser != None else None
+    if loguser != None:
+        print(f'Logs for user {loguser}:')
     logs = _queryLogRecords(logUserId,limit=limit,whereClause=whereClause)
     logs = utils.deleteNulls(logs,systemFields=False)
     logs1 = []
@@ -289,12 +293,21 @@ def createContext():
     context['firstLineOut'] = True
     return context
 
+def parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=False,error=False,limits=False):
+    try:
+       return  _parse(logId,logData,printLimits,userDebug,level,writeTofile,error,limits)
+    except KeyboardInterrupt as e:
+        print(f"Parsing for logI {logId} interrupted.")
+    except Exception as e:
+        raise 
 
-def parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=False,error=False):
+def _parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=False,error=False,limits=False):
     context = createContext()
     context['printLimits'] = printLimits
     context['userDebug'] = userDebug
     context['logLevel'] = level
+    context['logId'] = logId
+    context['limits'] = limits
 
     context['lines'] = logData.splitlines()
     debugList = []
@@ -307,6 +320,8 @@ def parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=F
 
         chunks = line.split('|')
         context['chunks'] = chunks
+        context['lenChunks'] = len(chunks)
+
         context['line'] = line
         context['num'] = num
 
@@ -328,27 +343,53 @@ def parse(logId,logData,printLimits=True,userDebug=True,level=None,writeTofile=F
                 continue
 
 
-        if len(chunks)>1 and chunks[1] in ['HEAP_ALLOCATE','STATEMENT_EXECUTE','VARIABLE_SCOPE_BEGIN']:
+
+        if context['lenChunks'] == 1:
+            continue
+
+        if len(chunks)>1 and chunks[1] in ['HEAP_ALLOCATE','STATEMENT_EXECUTE','VARIABLE_SCOPE_BEGIN','HEAP_ALLOCATE','SYSTEM_METHOD_ENTRY','SYSTEM_METHOD_EXIT','SOQL_EXECUTE_EXPLAIN']:
             continue
 
         if '|SYSTEM_MODE_EXIT|' in line:
             nop=1
 
-        parseUserInfo(context)
-        parseExceptionThrown(context)
-        parseVariableAssigment(context)
-        parseLimits(context) 
-        parseUserDebug(context)
+        if parseVariableAssigment(context):
+            continue
+
+        if parseLimits(context):
+            continue
+        if parseUserDebug(context):
+            continue
+        if parseUserInfo(context):
+            continue
+      #  parseUserDebug(context)
+        if parseExceptionThrown(context):
+            continue
+
+        if parseSOQL(context)==True:
+            continue
+
+        if parseMethod(context):
+            continue
+
+        if parseDML(context):
+            continue
+
+        if parseConstructor(context):
+            continue
+        if parseCodeUnit(context):
+            continue
+        if parseConstructor(context):
+            continue
+
         parseNamedCredentials(context)
         parseCallOutResponse(context)
         parseWfRule(context)
-        parseConstructor(context)
         parseFlow(context)
-        parseCodeUnit(context) 
-        parseDML(context)
-        parseSOQL(context)
-        parseMethod(context)
+        
 
+    if len(context['openItemsList']) > 0:
+        a=1
     appendEnd(context)
     context['debugList'] = processRepetition(debugList)
     printDebugList(context,logId,writeTofile,error=error)
@@ -371,7 +412,9 @@ def decreaseIdent_pop_setFields(context,type,value,key='key',endsWith=None,decre
 def parseUserInfo(context):
     if '|USER_INFO|' in context['line']:
         obj = setTimes(context,context['line'],field='output',value=context['chunks'][4],type='USER_INFO')
-        context['openItemsList'].append(obj)
+        context['debugList'].append(obj)
+        return True
+    return False
 #312780516
 def appendEnd(context):
 
@@ -386,15 +429,18 @@ def appendEnd(context):
 def parseSOQL(context):
     line = context['line']
     chunks = context['chunks']
-
-    if '|SOQL_EXECUTE_BEGIN|' in line:
+    if _isInOp(context,'SOQL_EXECUTE_BEGIN'):
+  #  if context['lenChunks']>1 and chunks[1] == 'SOQL_EXECUTE_BEGIN':
+   # if '|SOQL_EXECUTE_BEGIN|' in line:
         obj = setTimes(context,line,type="SOQL")
         obj['query'] = chunks[4]
         obj['object'] = chunks[4].lower().split(' from ')[1].strip().split(' ')[0]
         obj['apexline'] = chunks[2][1:-1]
         append_and_increaseIdent(context,obj,increase=False)
+        return True
 
-    if '|SOQL_EXECUTE_END|' in line:
+    if context['lenChunks']>1 and chunks[1] == 'SOQL_EXECUTE_END':
+ #   if '|SOQL_EXECUTE_END|' in line:
         context['totalQueries'] = context['totalQueries'] + 1
         obj = decreaseIdent_pop_setFields(context,type="SOQL",key='type',value='SOQL',decrease=False)
         obj['rows'] = chunks[3].split(':')[1]
@@ -404,17 +450,55 @@ def parseSOQL(context):
         _from = _from.split(' ')[0]
 
         obj['output'] = f"Select: {_from} --> {obj['rows']} rows"
+        return True
 
+    return False
 def parseLimits(context):
     line = context['line']
     chunks = context['chunks'] 
 
+    if '|LIMIT_USAGE|' in line:
+        if '|SOQL|' in line:
+            context[f'DEF:SOQL queries'] = chunks[4]
+
+    if '|LIMIT_USAGE_FOR_NS|' in line:
+        obj = setTimes(context,line,type='LIMIT')
+        obj['output'] = f"{chunks[1].lower()}  {chunks[1]}"
+        context['debugList'].append(obj)
+
+        limits = chunks[2]
+        if limits == '(default)':
+            limitsNS = 'DEF:'
+        elif limits == 'vlocity_cmt':
+            limitsNS ='CMT:'
+        else:
+            limitsNS = f"{limits}:"
+
+        next = 1
+        nextline = context['lines'][context['num']+next]
+        while '|' not in nextline:
+            if 'SOQL queries' in nextline:
+                nlchunks = nextline.split(' ')
+                if int(context[f'{limitsNS}SOQL queries']) < int(nlchunks[6]):
+                    context[f'{limitsNS}SOQL queries'] = nlchunks[6]
+            if 'CPU time' in nextline:
+                nlchunks = nextline.split(' ')
+
+                if int(context[f'{limitsNS}CPU time']) < int(nlchunks[5]):
+                    context[f'{limitsNS}CPU time'] = nlchunks[5]
+            next = next + 1
+            nextline = context['lines'][context['num']+next]
+
+def parseLimitsX(context):
+    line = context['line']
+    chunks = context['chunks'] 
+
     if '*** getCpuTime() ***' in line:
         chs = chunks[4].split(' ')
         context[f'DEF:CPU time'] = chs[4]
     if 'CPU Time:' in line:
         chs = chunks[4].split(' ')
-        context[f'DEF:CPU time'] = chs[2]      
+        context[f'DEF:CPU time'] = chs[2]    
     if '*** getQueries() ***' in line:
         chs = chunks[4].split(' ')
         context[f'DEF:SOQL queries'] = chs[4]
@@ -432,6 +516,7 @@ def parseLimits(context):
         else:
             context['limits_NS'] = f"{limits}:"
 
+
         limitsDict = setTimes(context,line,type='LIMITS')
         limitsDict['limitType'] = limits
         limitsDict['output'] = limitsDict['limitType']
@@ -461,11 +546,49 @@ def parseUserDebug(context):
     line = context['line']
     chunks = context['chunks']
 
+    if '|USER_DEBUG|' in line:
+        obj = setTimes(context,line,type='DEBUG')
+        obj['type'] = 'DEBUG'
+        obj['subType'] = chunks[3]
+        obj['string'] = chunks[4]
+        obj['apexline'] = chunks[2][1:-1]
+        obj['output'] = obj['string'] 
+        context['debugList'].append(obj)
+        next = 1
+        nextline = context['lines'][context['num']+next]
+        while '|' not in nextline:
+            obj = context['debugList'][-1].copy()
+            context['debugList'].append(obj)
+            obj['string'] = nextline
+            obj['output'] = nextline
+            next = next + 1
+            nextline = context['lines'][context['num']+next]
+
+        if '*** getCpuTime() ***' in line:
+            chs = chunks[4].split(' ')
+            context[f'DEF:CPU time'] = chs[4]
+        if 'CPU Time:' in line:
+            chs = chunks[4].split(' ')
+            context[f'DEF:CPU time'] = chs[2]    
+        if '*** getQueries() ***' in line:
+            chs = chunks[4].split(' ')
+            context[f'DEF:SOQL queries'] = chs[4]
+
+        return True
+
+    return False
+def parseUserDebugX(context):
+    line = context['line']
+    chunks = context['chunks']
+
     if '|' in line:   #This is a new line always. 
+        if 'debug_multiLine' in context and context['debug_multiLine'] == True:
+            a=1
         context['debug_multiLine'] = False
 
     if '|USER_DEBUG|' in line:
         context['debug_multiLine'] = True
+
     if 'debug_multiLine' in context and context['debug_multiLine'] == True:
         if '|' in line:
             obj = setTimes(context,line,type='DEBUG')
@@ -483,40 +606,55 @@ def parseUserDebug(context):
         obj['output'] = obj['string'] 
 
 def parseExceptionThrown(context):
+
     line = context['line']
     chunks = context['chunks']
 
-    if '|' in line:   #This is a new line always. 
-        context['exception_multiline'] = False
-
-    if context['exception'] == True:
-        if context['exception_multiline'] == True:   #This is a new line without the exeption
-            if line != '':
-                obj = context['debugList'][-1].copy()
-                obj['output'] = line
-                context['debugList'].append(obj)
-
-           #     obj['lines'].append(line)
-
-    if '|EXCEPTION_THROWN|' in line:
+    if context['lenChunks']>1 and chunks[1] == 'EXCEPTION_THROWN':
         obj = setTimes(context,line,type='EXCEPTION',field='output',value=chunks[3])
         context['exception'] = True
-        context['exception_multiline'] = True
         context['debugList'].append(obj)
         context['file_exception'] = True
+        next = 1
+        nextline = context['lines'][context['num']+next]
+        while '|' not in nextline:
+            obj = context['debugList'][-1].copy()
+            context['debugList'].append(obj)
+            obj['output'] = nextline
+            next = next + 1
+            nextline = context['lines'][context['num']+next]
+        return True
 
-    if '|FATAL_ERROR|' in line:
+    if context['lenChunks']>1 and chunks[1] == 'FATAL_ERROR':
         obj = setTimes(context,line,type='EXCEPTION',field='output',value=chunks[2])
         context['exception'] = True
-        context['exception_multiline'] = True
         context['debugList'].append(obj)
         context['file_exception'] = True
-
+        next = 1
+        nextline = context['lines'][context['num']+next]
+        while '|' not in nextline:
+            obj = context['debugList'][-1].copy()
+            context['debugList'].append(obj)
+            obj['output'] = nextline
+            next = next + 1
+            nextline = context['lines'][context['num']+next]
+        return True
+    return False
+def _isInOp(context,text,contains=False):
+    if context['lenChunks']<2:
+        return False
+    if contains:
+        if text in context['chunks'][1]:
+            return True
+    elif context['chunks'][1] == text:
+        return True
+    return False
 def parseWfRule(context):
-    workflow = {}
     line = context['line']
     chunks = context['chunks'] 
-    if '|WF_RULE_EVAL' in line:
+
+    if _isInOp(context,'WF_RULE_EVAL',contains=True):
+  #  if '|WF_RULE_EVAL' in line:
         if 'BEGIN' in chunks[1]:
             obj = setTimes(context,line,field='output',value='Workflow',type='RULE_EVAL')
             append_and_increaseIdent(context,obj)
@@ -526,12 +664,14 @@ def parseWfRule(context):
         #    obj = getFromList(context['openItemsList'],'output','Workflow')
         #    setTimes(line,obj)
 
-    if '|WF_CRITERIA_' in line:
+    if _isInOp(context,'WF_CRITERIA',contains=True):
+  #  if '|WF_CRITERIA_' in line:
         if 'BEGIN' in chunks[1]:
             obj = setTimes(context,line,type='WF_CRITERIA')
             obj['nameId'] = chunks[2]
             obj['rulename'] = chunks[3]
             obj['rulenameId'] = chunks[4]
+            obj['output'] = obj['rulename']
 
             append_and_increaseIdent(context,obj)
 
@@ -539,15 +679,22 @@ def parseWfRule(context):
             obj =decreaseIdent_pop_setFields(context,type='WF_CRITERIA',key='type',value='WF_CRITERIA')   
             obj['result'] = chunks[2]
             obj['output'] = f"{obj['rulename']} --> {obj['result']}"
+  
+    if _isInOp(context,'WF_RULE_NOT_EVALUATED'):
+  #  if 'WF_RULE_NOT_EVALUATED' in line:
+        obj =decreaseIdent_pop_setFields(context,type='WF_CRITERIA',key='type',value='WF_CRITERIA')   
+        obj['output'] = f"{obj['rulename']} --> Rule Not Evaluated"
 
-    if '|WF_ACTION|' in line:
+    if _isInOp(context,'WF_ACTION'):
+ #   if '|WF_ACTION|' in line:
         obj = getFromList(context['openItemsList'],'output','Workflow',delete=False)
         obj['action'] = chunks[2]
 
 def parseMethod(context):
     line = context['line']
     chunks = context['chunks'] 
-    if '|METHOD_' in line:
+    if context['lenChunks']>1 and '|METHOD_' in  chunks[1]:
+  #  if '|METHOD_' in line:
         if len(chunks)<4:
             print(line)
             return
@@ -567,6 +714,7 @@ def parseMethod(context):
             else:
                 context['openItemsList'].append(obj)
                 increaseIdent(context)
+            return True
         else:
             obj = getFromList(context['openItemsList'],'method',method)
             if obj == None:
@@ -586,7 +734,8 @@ def parseMethod(context):
                 obj['output']=obj['method']
             else:
                 obj['output']=obj['Id']
-            
+            return True
+        return False
 def parseVariableAssigment(context):
     line = context['line']
     chunks = context['chunks'] 
@@ -604,7 +753,9 @@ def parseVariableAssigment(context):
 
         else:
             context['EXP_VAR'] = False
-    if '|VARIABLE_ASSIGNMENT|' in line:
+        return True
+
+    if _isInOp(context,'VARIABLE_ASSIGNMENT'):
         if len(chunks) >= 5:
 
             if 'ExecutionException' in chunks[4] or 'ExecutionException' in chunks[4]:
@@ -618,12 +769,14 @@ def parseVariableAssigment(context):
                 obj['output'] = obj['string'] 
 
                 context['EXP_VAR'] = True
-
+        return True
+    return False
 def parseDML(context):
     line = context['line']
     chunks = context['chunks']
 
-    if '|DML_BEGIN|' in line:
+    if _isInOp(context,'DML_BEGIN'):
+ #   if '|DML_BEGIN|' in line:
         obj = setTimes(context,line,type="DML")
         obj['OP'] = chunks[3]
         obj['Type'] = chunks[4]
@@ -631,15 +784,19 @@ def parseDML(context):
         obj['apexline'] = chunks[2][1:-1]
         obj['output'] = f"{obj['OP']} {obj['Type']}" 
         append_and_increaseIdent(context,obj)
+        return True
 
-    if '|DML_END|' in line:
+    if _isInOp(context,'DML_END'):
+  #  if '|DML_END|' in line:
         decreaseIdent_pop_setFields(context,'DML',key='Id',value=chunks[2])
-
+        return True
+    return False
 def parseCallOutResponse(context):
     line = context['line']
     chunks = context['chunks']
 
-    if 'CALLOUT_RESPONSE' in line:
+    if _isInOp(context,'CALLOUT_RESPONSE'):
+ #   if 'CALLOUT_RESPONSE' in line:
         obj = setTimes(context,line,type='CALLOUT')
         #  obj['type'] = 'DEBUG'
         #  obj['subType'] = chunks[3]
@@ -653,49 +810,63 @@ def parseConstructor(context):
     line = context['line']
     chunks = context['chunks']
 
-    if '|CONSTRUCTOR_ENTRY|' in line:
+    if _isInOp(context,'CONSTRUCTOR_ENTRY'):
+  #  if '|CONSTRUCTOR_ENTRY|' in line:
         obj = setTimes(context,line,field='output',value=chunks[5],type='CONSTRUCTOR')
         append_and_increaseIdent(context,obj)
+        return True
 
-    if '|CONSTRUCTOR_EXIT|' in line:
+    if _isInOp(context,'CONSTRUCTOR_EXIT'):
+  #  if '|CONSTRUCTOR_EXIT|' in line:
         decreaseIdent_pop_setFields(context,type='CONSTRUCTOR',key='output',value=chunks[5])
-
+        return True
+    return False
 def parseCodeUnit(context):
     line = context['line']
     chunks = context['chunks']
 
-    if '|CODE_UNIT_STARTED|' in line:
+    if _isInOp(context,'CODE_UNIT_STARTED'):
+  #  if '|CODE_UNIT_STARTED|' in line:
         obj = setTimes(context,line,type='CODE_UNIT')
         obj['output'] = chunks[4] if len(chunks)>4 else chunks[3]
         append_and_increaseIdent(context,obj)
+        return True
 
-    if '|CODE_UNIT_FINISHED|' in line:
+    if _isInOp(context,'CODE_UNIT_FINISHED'):
+  #  if '|CODE_UNIT_FINISHED|' in line:
         decreaseIdent_pop_setFields(context,'CODE_UNIT',key='output',value=chunks[2])
-
+        return True
+    return False
 def parseNamedCredentials(context):
     line = context['line']
     chunks = context['chunks']
 
-    if '|NAMED_CREDENTIAL_REQUEST|' in line:
+    if _isInOp(context,'NAMED_CREDENTIAL_REQUEST'):
+ #   if '|NAMED_CREDENTIAL_REQUEST|' in line:
         obj = setTimes(context,line,field='output',value=chunks[2],type='NAMED_CRD')
         append_and_increaseIdent(context,obj)
+        return True
 
-    if "|NAMED_CREDENTIAL_RESPONSE|" in line:
+    if _isInOp(context,'NAMED_CREDENTIAL_RESPONSE'):
+ #   if "|NAMED_CREDENTIAL_RESPONSE|" in line:
         obj = decreaseIdent_pop_setFields(context,type='NAMED_CRD',key='type',value='NAMED_CRD')
-
+        return True
+    return False
 def parseFlow(context):
     line = context['line']
     chunks = context['chunks']
     debugList = context['debugList']
 
-    if '|FLOW_START_INTERVIEWS_BEGINxx|' in line:
-        obj = setTimes(context,line,type='FLOW_START_INTERVIEWS',field='output',value='FLOW_START_INTERVIEWS')
-        append_and_increaseIdent(context,obj)
+    if 1==2:
+        if '|FLOW_START_INTERVIEWS_BEGINxx|' in line:
+            obj = setTimes(context,line,type='FLOW_START_INTERVIEWS',field='output',value='FLOW_START_INTERVIEWS')
+            append_and_increaseIdent(context,obj)
 
-    if '|FLOW_START_INTERVIEWS_ENDxx|' in line:
-        decreaseIdent_pop_setFields(context,'FLOW_START_INTERVIEWS',key='output',value='FLOW_START_INTERVIEWS')
+        if '|FLOW_START_INTERVIEWS_ENDxx|' in line:
+            decreaseIdent_pop_setFields(context,'FLOW_START_INTERVIEWS',key='output',value='FLOW_START_INTERVIEWS')
 
-    if '|FLOW_START_INTERVIEW_BEGIN|' in line:
+    if _isInOp(context,'FLOW_START_INTERVIEW_BEGIN'):
+  #  if '|FLOW_START_INTERVIEW_BEGIN|' in line:
         obj = setTimes(context,line,type='FLOW_START_INTERVIEW')
         obj['Id'] = chunks[2]
         obj['Name'] = chunks[3]
@@ -703,11 +874,12 @@ def parseFlow(context):
         obj['key'] = obj['Id']
         append_and_increaseIdent(context,obj)
 
-    if '|FLOW_START_INTERVIEW_END|' in line:
+    if _isInOp(context,'FLOW_START_INTERVIEW_END'):
+ #   if '|FLOW_START_INTERVIEW_END|' in line:
         decreaseIdent_pop_setFields(context,'FLOW_START_INTERVIEW',value=chunks[2])
 
-
-    if '|FLOW_ELEMENT_ERROR|' in line:
+    if _isInOp(context,'FLOW_ELEMENT_ERROR'):
+  #  if '|FLOW_ELEMENT_ERROR|' in line:
         obj = setTimes(context,line,type='FLOW_ELEMENT_ERROR')
         obj['message'] = chunks[2]
         obj['elementType'] = chunks[3]
@@ -763,10 +935,13 @@ def _printDebugList(context,logId,error=False):
                 firstLines = False
                 print()
                 print()
-        if '*** getCpuTime() ***' in d['output']:
-            continue
-        if '*** getQueries() ***' in d['output']:
-            continue
+        if context['limits'] == False:
+            if '*** getCpuTime() ***' in d['output']:
+                continue
+            if '*** getQueries() ***' in d['output']:
+                continue
+            if d['type'] == 'LIMIT':
+                continue
         printParsedLog(context,d)    
     print()
 def processRepetition(debugList):
@@ -981,7 +1156,11 @@ def printParsedLog(context,d):
         print(f"{'time(ms)':10}|{'elapsed':10}|{'time1(ns)':12}|{'time2(ns)':12}|{'t2-t1':10}|{'Qd':4}|{'Qt':4}|{'cpuD':6}|{'CPUin':6}|{'Q':3}|{'Qcm':3}|{'type':21}|{'al':4}{'':50}")
         context['firstLineOut'] = False
 
-    print(f"{ms(d['elapsedTime']):10}|{_elapsedPrevD:8}|{d['timeStamp'][0]:12}|{_timeStamp1:12}|{_delta:>10}|{_totalQueriesD:4}|{_totalQueries1:4}|{_cpuPrevD:6}|{_cpuTime0:>6}|{_sql2:>3}|{_sqlcmt2:>3}|{_typeColor}{_type:21}{utils.CEND}|{_apexline:>4}| {val[:150]:50}"+utils.CEND)
+    if _type not in ['EXCEPTION','DEBUG-ERROR']:
+        val = val[:150]
+    if _type == 'DEBUG-ERROR':
+        a=1
+    print(f"{ms(d['elapsedTime']):10}|{_elapsedPrevD:8}|{d['timeStamp'][0]:12}|{_timeStamp1:12}|{_delta:>10}|{_totalQueriesD:4}|{_totalQueries1:4}|{_cpuPrevD:6}|{_cpuTime0:>6}|{_sql2:>3}|{_sqlcmt2:>3}|{_typeColor}{_type:21}{utils.CEND}|{_apexline:>4}| {val:50}"+utils.CEND)
 
 def ms(val):
     return f"{val/1000000:10.2f}"
