@@ -278,7 +278,8 @@ def _checkAndSetConnectionIfExists(connectionName):
     for con in _initializedConnections:
         if con['connectionName'] == connectionName:
             _threadContext(connectionName)
-            glog().info(f"Connection {connectionName} set.")
+            if connectionName != 'ConnectionLess':
+                glog().info(f"Connection {connectionName} set.")
             return True
     return False
 
@@ -305,8 +306,8 @@ def _initMain(name,url,token=None,input=None,output=None,debug=None,username=Non
         connection['username'] = username
     _initializedConnections.append(connection)
     _threadContext(connection['connectionName'])
-
-    glog().info(f"Connection {connection['connectionName']} initialized.")
+    if name != 'ConnectionLess':
+        glog().info(f"Connection {connection['connectionName']} initialized for username {username}.")
 
 def getCurrentThreadConnection():
     """
