@@ -160,7 +160,7 @@ class Test_Main(unittest.TestCase):
             print(traceback.format_exc())
             self.assertTrue(1==2)
 
-    def test_logs_userDefault(self):
+    def test_logs_userDefault_loguser(self):
         try:
             InCli._main(["-default:set","loguser","Alias:ana.r"])
             InCli._main(["-u","NOSDEV","-logs"])
@@ -195,18 +195,34 @@ class Test_Main(unittest.TestCase):
    
     def test_log_ID_2(self):
         try:
-            id = '07L3O00000Dh64qUAB'
-          #  id = None
+            id = '07L3O00000Dh95XUAR'
+      #      id = None
 
             if id == None:
                 restClient.init('NOSDEV')
                 id = query.queryField("Select Id FROM ApexLog order by StartTime desc limit 1")
             InCli._main(["-u","NOSDEV","-logs",id])
+
         except Exception as e:
             print(e)
             print(traceback.format_exc())
             self.assertTrue(1==2)
 
+    def test_log_limitinfo(self):
+        try:
+           # id = '07L3O00000Dh69HUAR'
+            id = None
+
+            if id == None:
+                restClient.init('NOSDEV')
+                id = query.queryField("Select Id FROM ApexLog order by StartTime desc limit 1")
+            InCli._main(["-u","NOSDEV","-logs",id])
+            InCli._main(["-u","NOSDEV","-logs",id,"-limitinfo"])
+
+        except Exception as e:
+            print(e)
+            print(traceback.format_exc())
+            self.assertTrue(1==2)
     def test_log_IDflow(self): #flow
         try:
             InCli._main(["-u","NOSDEV","-logs","-inputfile","/Users/uormaechea/Documents/Dev/python/InCliLib/InCLipkg/test/files/flowAndWF.log"])
@@ -216,7 +232,7 @@ class Test_Main(unittest.TestCase):
 
     def test_log_ID(self):
         try:
-            InCli._main(["-u","NOSDEV","-logs","07L3O00000Dh8gPUAR"])
+            InCli._main(["-u","NOSDEV","-logs","-last","100"])
         except Exception as e:
             print(e)
             print(traceback.format_exc())
@@ -316,7 +332,7 @@ class Test_Main(unittest.TestCase):
             InCli._main(["-default:set","u"])
             InCli._main(["-default:set","u","NOSDEV"])
             InCli._main(["-default:get","u"])        
-            InCli._main(["-logs","-last","1"])
+            res = InCli._main(["-logs","-last","1"])
             InCli._main(["-default:del","u"])
         except Exception as e:
             utils.printException(e)
