@@ -26,7 +26,8 @@ class Test_Parse(unittest.TestCase):
       #  objs = jsonFile.read('/Users/uormaechea/Documents/Dev/python/VirtualEnvs/prj1/.incli/logs/07L3O00000Dg1OSUAZ')
 
       try:
-        parsed = debugLogs.parseLog(filepath='/Users/uormaechea/Documents/Dev/python/VirtualEnvs/prj1/.incli/logs/07L3O00000Dg1OSUAZ.log')
+        logId = '07L3O00000Dg1OSUAZ'
+        parsed = debugLogs.parseLog(filepath=f'/Users/uormaechea/Documents/Dev/python/VirtualEnvs/prj1/.incli/logs/{logId}.log')
 
         repss = utils.repeatingSequence(parsed,"output")
 
@@ -36,14 +37,22 @@ class Test_Parse(unittest.TestCase):
           if isrepe == True and obj == None:
             continue
           parsedX.append(parsedLine)
-        debugLogs.printDebugList(parsedX)    
+        debugLogs.printDebugList(parsedX,logId)    
       
 
       except Exception as e:
         print(e)
         traceback.format_exc()
 
+    def test_parseStorage(self):
+      try:
+        restClient.init('ConnectionLess')
+        debugLogs._parseStorage()
+      except Exception as e:
+        print(e)
+        traceback.format_exc()
 
+      print()
         
 
 
